/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
/******************************************************************************
 *  readtool.c   file reading routines, using buffers                         *
 ******************************************************************************/
#include <config.h>
#include <stdio.h>
#include <string.h>
#include <netinet/in.h>
#include <assert.h>
#include "vector.h"

#define OBJBASELN 67

/* read chunk into memory */
int FGetObj(void *file, MapObject * object, long offset)
{
    char *obj, len1, len2;

    obj = (char *)(file+offset);
    memcpy(&object->length, obj, 4);

    object->length = ntohl(object->length);

    memcpy(&object->owner, obj + 4, 4);
    object->owner[4] = 0;
    memcpy(&object->modifier, obj + 8, 4);
    object->modifier[4] = 0;
    memcpy(&object->location, obj + 12, 5);
    object->location[5] = 0;
    memcpy(&object->modsec, obj + 17, 8);	/* includes flags */
    object->type = obj[25] & 0x7f;
    object->link = obj[25] >> 7;
    memcpy(&object->id, obj + 26, 28); /*includes bounds and digscale */ ;
    memcpy(&object->cfcc, obj + 54, 4);
    object->cfcc[4] = 0;
    memcpy(&object->place, obj + 58, 9); /* includes stco and alias */ ;

    object->stco = ntohl(object->stco);

    object->multiples = object->place >> 31;
    object->place &= 0x7FFFFFFF;
    len1 = obj[OBJBASELN];
    len2 = obj[OBJBASELN + (int) len1 + 1];
    object->offset2 = offset + OBJBASELN + 2 + (int) len1 + (int) len2;
    return 0;
}

int FGetObjName(char * file, long offset, char *prefix, char *name, short *space)
{
    char len1, len2;

    offset += OBJBASELN;

    file += offset;
    len1 = *file++;
    if ((int) len1 > 0) {
	memcpy(prefix, file, (int)len1);
    }
    prefix[(int) len1] = 0;

    file+=len1;
    len2 = *file++;

    if ((int) len2 > 0) {
	memcpy(name, file, (int)len2);
    }
    name[(int) len2] = 0;

    if (space)
	*space = ((int) len1 + (int) len2) - (strlen(prefix) + strlen(name));
    return 0;
}

#if 0
int FGetObjText(char * file, long offset2, char *text, short *space)
{
    short len = 0; // this is a crutch, this code is broken and never worked.
    long length;

    offset2 += 4;
    file += offset2;
    length = min(255, len);
    
    assert(length < 1000);
    memcpy(text, file, length);
    text[length] = 0;
    if (space)
	*space = length - strlen(text);
    return 0;
}
#endif


int FGetObjNPts(char * file, long offset2, long *npts)
{
    offset2 += 2;

    memcpy(npts,file+offset2,4);

    return 0;
}

/* read entire point string into memory at once (hopefully faster) */
int FGetObjPtBuf(char * file, long offset, long size, unsigned char *buffer)
{
    extern long objsize;

    if ( offset+size > objsize ) {
	fprintf(stderr,"file size = %ld, offset = %ld, size = %ld\n", objsize,offset,size);
    }
    memcpy(buffer,file+offset,size);
    return 0;
}

int FGetObjPtfromBuf(unsigned char *buf, long *pos, char *action, long *lat,
		     long *lon)
{
    long lonact;

    long latact;

    short flags;
    memcpy(&lonact, (buf + *pos), 4);

    lonact = ntohl(lonact);

    *lon = lonact & 0x7FFFFFFF;
    *action = (lonact & 0x80000000) ? FROM : TO;

    memcpy(&latact, (buf + *pos + 4), 4);
    *lat = ntohl(latact);

    memcpy(&flags, (buf + *pos + 8), 2);

    flags = ntohs(flags);

    *pos += 10 + ShortBits(flags) * 4;
    return 0;
}
