/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
/******************************************************************************
 * tigerwww.h - main header file (types, constants, declarations)             *
 ******************************************************************************/
#include <X11/Xos.h>
#include <X11/cursorfont.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/MainW.h>
#include <Xm/LabelG.h>
#include <X11/xpm.h>

#include <stdio.h>

/********** CONSTANTS **********/
#define MAXDATASETS 16
#define MAXLAYERS   25
#define MAXFEATURES 100
#define MAX_COLORS  256
#define MAXMARKERS  256
#define MAXPOINTS   15000
#define AVGPPI      75

#define FALSE       0
#define TRUE        1
#define MOVE        1
#define DRAW        0
#define FROM        1
#define TO          0
#define WHITE       0
#define BLACK       1

/********** PROJECTIONS **********/
#define MERCATOR	1
#define ORTHOGRAPHIC	2

/* Object Types */
enum {
    RECT = 1, ELLIPSE, POINT, SYMBOL = 3, LINESTRING, POLYGON, TEXT, PICT, ALIAS
};

#define DEF_W       -77.2
#define DEF_E       -76.8
#define DEF_N       39
#define DEF_S       38.77
#define DEF_DEBUG   0
#define UNDEF_DEG   -1000
#define NO_GRID     -1
#define DEF_GRID    0
#define DEF_GRID_SP 150		/* display parallels about every 150 pixels */

/********** MACROS *************/
#define MBR_WIDTH(rect) ((rect).ELon-(rect).WLon)
#define MBR_HEIGTH(rect) ((rect).NLat - (rect).SLat)
#define abs(val)    (val < 0 ? -(val) : val )
#define min(val1,val2) (val1 < val2 ? val1 : val2 )
#define max(val1,val2) (val1 > val2 ? val1 : val2 )
#define DATELN(rect) ( ((rect).ELon>(rect).WLon)-((rect).WLon>(rect).ELon) )

/*********** TYPES *************/
/* container for settings for each layer in the map */
typedef struct {
    short r;
    short g;
    short b;
    short w;
    short color;
    char file[10];
    GC brush;
} Symbol;

typedef struct {
    char name[30];
    char label[20];
    int on;
    Symbol line;
    Symbol fill;
    Symbol point;
} Config;

/* type for very long numbers (i.e. object ID's) */
typedef struct {
    long hi;
    long lo;
} long64;

typedef struct {		/* coordinates for Minimum Bounding Rectangles */
    long WLon;
    long SLat;
    long ELon;
    long NLat;
} MBR;

typedef struct {
    long offset;
    MBR bounds;
    long64 id;
    char name[4];
} SummaryRec;

typedef struct LPoint {		/* an attributed lat/lon vertex */
    long lat;
    long lon;
    char action;
    unsigned short flags;
    long attrib[16];
} LPoint;

typedef struct Place {
    long lat;
    long lon;
    long pop;
    char len;
    char st[2];
    char name[30];
} Place;

typedef struct Rectangle {
    int xl;
    int yt;
    int xr;
    int yb;
} Rectangle;

typedef struct Thematic {
    int on;
    char level[10];
    char var[10];		/* variable to be displayed */
    char meth[10];		/* classification method (quantiles/eq interval) */
    int hid;			/* digits to extract from id # */
    int xd;
    int idd;
} Thematic;

/* Container for object attributes to be read from OBJ file */
typedef struct {
    long length;		/* +0 */
    char owner[5];		/* +4 */
    char modifier[5];		/* +8 */
    char location[6];		/* +12 */
    unsigned long modsec;	/* +17 */
    long flags;			/* +21 */
    char type;
    char link;			/* +25 */
    long64 id;			/* +26 */
    MBR bounds;			/* +34 */
    long digscale;		/* +50 */
    char cfcc[4];		/* +54 */
    long place;			/* +57 */
    char multiples;
    long stco;			/* +61 */
    char alias;			/* +65 */
    long offset2;
} MapObject;

/* container for settings for the requested map image */
typedef struct MapStruct {
    int proj;

    /* parameters specified by user */
    int GridSpace;				/* spacing of grid */
    int debug;					/* flag for debugging info */
    float CtrLonD, CtrLatD;			/* position of map (degrees) */
    float minWidthD, maxWidthD, HeightD;	/* width and height (degrees) */
    int WidthP, HeightP;			/* width and height of image */
    char forceon[120], forceoff[120];		/* layers/datasets to turn on or off */

    /* calculated parameters */
    long CtrLatU, CtrLonU;			/* position of map (in LU) */
    long HalfWidU;				/* clipping width of map (projected MU) */
    MBR BoundsU, nBoundsU;			/* map bounds (lu), nBoundsU is normalized */
    float EdgeSD, EdgeND, EdgeWD, EdgeED;	/* map bounds (deg) */
    long muPerPix, scale;			/* scale of map image */
    int GenShift;				/* bits to reduce precision */
    Config *datasets;				/* dataset preferences */
    short ndirs;
    Config *layers;				/* feature layer preferences */
    short nlyrs;
    Config *features;				/* feature preferences */
    short nfeats;
    XPoint pointlist[MAXPOINTS];		/* list of pixel coords for polygons */
    MapObject object;
    Thematic theme;				/* parameters for thematic map */
} MapStruct;

extern char *vmap_root;

/********** FUNCTION PROTOTYPES ************/
/* functions in drawtool.c */
void DrawGraticule(Pixmap image, MapStruct * ms, GC gc);
void AllocColor(Pixmap image, Symbol * sym);
/*void AllocColors(Pixmap image, Config *temp); */
void DrawMap(MapStruct * ms, Pixmap image);
void DrawDatasets(Pixmap image, MapStruct * ms);
void DrawLayers(Pixmap image, MapStruct * ms, char *dir);
void DrawObjects(Pixmap image, MapStruct * ms, char *basename, Config * config);
void DrawPolyline(Pixmap image, MapStruct * ms, char *objfile, SummaryRec * s, Symbol * style);
void DrawPolygon(Pixmap image, MapStruct * ms, char *objfile, SummaryRec * s, Symbol * linesym, Symbol * fillsym);
void DrawPoint(Pixmap image, MapStruct * ms, SummaryRec * s, Symbol * style);
void DrawMarkers(Pixmap image, MapStruct * ms);
void DrawGlobe(Pixmap image, MapStruct * ms, GC gc);

/* functions in stattool.c */
void StDrawObjects(Pixmap image, MapStruct * ms, char *basename, Config * config);

/* functions in label.c */
void LabelGraticule(Pixmap image, MapStruct * ms, GC gc);
void LabelPlaces(Pixmap image, MapStruct * ms, Config * config);

/* functions in legend.c */
void DrawLegend(MapStruct * ms);
void DrawScaleBar(MapStruct * ms, Pixmap image);
void DrawLegendItem(Pixmap image, Config * config, long scale, int *x, int *y);
void DrawThematicLegend(Pixmap image, MapStruct * ms, int layer, int *x, int *y);

/* functions in projtool.c */
void ComputeSine(unsigned short *xTable);
void Project(MapStruct * ms, long lat, long lon, int *x, int *y);
void nProject(MapStruct * ms, long lat, long lon, int *x, int *y);
void map_params(MapStruct * ms);

/* functions in tigerwww.c */
MapStruct *map_init();

/* functions in config.c */
void ReadConfig(MapStruct * ms, char *fn);

/* functions in readtool.c */
int FGetObj(void *file, MapObject * object, long offset);
int FGetObjName(char *file, long offset, char *prefix, char *name, short *space);
int FGetObjText(char *file, long offset2, char *text, short *space);
int FGetObjNPts(char *file, long offset2, long *npts);
int FGetObjPointAtt(char *file, long *offset2, LPoint * point);
int FGetObjPtBuf(char *file, long offset, long size, unsigned char *buffer);
int FGetObjPtfromBuf(unsigned char *buf, long *pos, char *action, long *lat,
		     long *lon);

/* functions in utils.c */
void Unpack(unsigned char packed, char *val1, char *val2);
short ShortBits(short s);
long Normalize(long x1, long x2);
int CenterMBR(MBR * rect);
char OverlapMBR(MBR rect1, MBR rect2);
char RectInRect(MBR rect1, MBR rect2);
char PtInRect(long x, long y, MBR rect);
char Sector(long x, long y, MBR rect);
char CodeInList(char *code, char *list);
int FeatureSym(MapStruct * ms, Symbol ** linesym, Symbol ** fillsym, Symbol ** ptsym, char *code);
